-------------------------------------------------------------------------------
-- Locals                                                                    --
-------------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("ClosetGnome")
local dewdrop = AceLibrary("Dewdrop-2.0")
local tablet = AceLibrary("Tablet-2.0")

local deequipQueue = nil
local queuedSet = nil
local lastEquippedSet = nil
local slotLocks = nil
local queuedForDelete = nil
local activeSlots = nil

local keybindingFrame = "ClosetGnomeKeybindingFrame"

local _G = getfenv(0)

local slots = {
	{slot = "Head"},
	{slot = "Shoulder"},
	{slot = "Chest"},
	{slot = "Waist"},
	{slot = "Legs"},
	{slot = "Feet"},
	{slot = "Wrist"},
	{slot = "Hands"},
	{slot = "SecondaryHand"},
	{slot = "MainHand"},
	{slot = "Ranged"},
	{slot = "Ammo"},
	{slot = "Neck"},
	{slot = "Back"},
	{slot = "Finger0"},
	{slot = "Finger1"},
	{slot = "Trinket0"},
	{slot = "Trinket1"},
	{slot = "Tabard", inactive = true},
	{slot = "Shirt", inactive = true},
	--{slot = "Relic"},
}

local NUM_QUIPS = 35

-------------------------------------------------------------------------------
-- Addon declaration                                                         --
-------------------------------------------------------------------------------

ClosetGnome = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")
ClosetGnome:RegisterDB("ClosetGnomeDB", "ClosetGnomePerCharDB")
ClosetGnome:RegisterDefaults("profile", {
	quips = true,
})
ClosetGnome:RegisterDefaults("char", {
	set = {},
	icons = {},
	keybindings = {},
	alwaysWeapons = true,
})
ClosetGnome.OnMenuRequest = {
	type = "group",
	name = L["ClosetGnome"],
	desc = L["ClosetGnome options."],
	args = {
		add = {
			type = "execute",
			name = L["Add"],
			desc = L["Creates a new set, or updates an existing one."],
			func = function() ClosetGnome:CreateSet() end,
			order = 101,
		},
		wear = {
			type = "group",
			name = L["Wear"],
			desc = L["Change equipment set."],
			args = {},
			order = 102,
			disabled = function() return next(ClosetGnome.OnMenuRequest.args.wear.args) == nil end
		},
		key = {
			type = "group",
			name = L["Keybinding"],
			desc = L["Assign a keybinding to a set."],
			args = {},
			order = 103,
			disabled = function() return next(ClosetGnome.OnMenuRequest.args.wear.args) == nil end
		},
		spacer = { type = "header", order = 104 },
		alwaysweapons = {
			type = "toggle",
			name = L["Always equip weapons"],
			desc = L["Equip weapons in sets even if you are in combat."],
			get = function() return ClosetGnome.db.char.alwaysWeapons end,
			set = function(v) ClosetGnome.db.char.alwaysWeapons = v end,
			order = 105,
		},
		quips = {
			type = "toggle",
			name = L["Quips"],
			desc = L["Toggle outputting random quips when equipping sets."],
			get = function() return ClosetGnome.db.profile.quips end,
			set = function(v) ClosetGnome.db.profile.quips = v end,
			order = 106,
		},
		delete = {
			type = "group",
			name = L["Delete"],
			desc = L["Delete a set."],
			args = {},
			order = 107,
			disabled = function() return next(ClosetGnome.OnMenuRequest.args.wear.args) == nil end
		},
	},
}
ClosetGnome:RegisterChatCommand({"/closetgnome", "/cg"}, ClosetGnome.OnMenuRequest, "CLOSETGNOME")

-- FuBar stuff.
ClosetGnome.hideWithoutStandby = true
local defaultIcon = "Interface\\Icons\\INV_Chest_Cloth_17"
ClosetGnome.hasIcon = defaultIcon
ClosetGnome.hasNoColor = true
ClosetGnome.defaultMinimapPosition = 240
ClosetGnome.clickableTooltip  = true

-------------------------------------------------------------------------------
-- Initialization                                                            --
-------------------------------------------------------------------------------

function ClosetGnome:OnInitialize()
	if not StaticPopupDialogs then
		StaticPopupDialogs = {}
	end
	StaticPopupDialogs["ClosetGnomeAdd"] = {
		text = L["|cffeda55fLeft-Click|r a slot to toggle it for this set. Green slots are |cff00ff00enabled|r, red are |cffff0000disabled|r.\n\nIf you want to assign an |cff0000fficon|r to this set, you can |cffeda55fCtrl-Click|r a slot to use that slots item as the icon.\n\nType the desired set name below and click Add when you are done."],
		button1 = L["Add"],
		button2 = L["Cancel"],
		OnCancel = function() self:CancelCreateSet() end,
		sound = "levelup2",
		whileDead = 1,
		hideOnEscape = 1,
		timeout = 0,
		OnShow = function()
			-- We have to do this onshow to reset the previous text
			_G[this:GetName().."EditBox"]:SetText("")
		end,
		OnAccept = function()
			local name = _G[this:GetParent():GetName().."EditBox"]:GetText()
			self:AddSetFromDoll(name)
		end,
		hasEditBox = 1,
	}
	StaticPopupDialogs["ClosetGnomeDelete"] = {
		text = L["Are you sure you want to delete the set %s?"],
		button1 = L["Delete"],
		button2 = L["Cancel"],
		showAlert = 1,
		timeout = 0,
		OnAccept = function() self:DeleteSet(queuedForDelete, true) end,
		OnCancel = function() queuedForDelete = nil end,
	}
end

function ClosetGnome:OnEnable()
	-- Create the button frames for keybindings
	for k, v in pairs(self.db.char.keybindings) do
		self:RegisterBinding(v, k)
	end

	if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
		self:AceEvent_FullyInitialized()
	end
	self:RegisterEvent("AceEvent_FullyInitialized")

	self:UpdateSetMenu()
end

function ClosetGnome:OnDisable()
	activeSlots = nil
	slotLocks = nil

	for k, v in pairs(self.db.char.keybindings) do
		self:UnsetBinding(k)
	end
end


-------------------------------------------------------------------------------
-- Keybindings                                                               --
-------------------------------------------------------------------------------

function ClosetGnome:ValidateBinding(combo)
	return AceLibrary("AceConsole-2.0").keybindingValidateFunc(combo)
end

function ClosetGnome:RegisterBinding(combo, set)
	if not combo or not set then return end

	if not self:ValidateBinding(combo) then
		self:Print(string.format(L["%s is not a valid keybinding."], "|cffd9d919"..combo.."|r"))
		return
	end

	local shouldPrint = true
	for k, v in pairs(self.db.char.keybindings) do
		if v == combo then
			shouldPrint = nil
		elseif k == set then
			self:UnsetBinding(set)
		end
	end

	if shouldPrint then
		self:Print(string.format(L["Registering keybinding %s to set %s."], "|cffd9d919"..combo.."|r", "|cffd9d919"..set.."|r"))
	end

	local comboCopy = combo
	local setCopy = set
	local frameName = keybindingFrame..comboCopy
	local frame = _G[frameName] or CreateFrame("Button", frameName, _G["UIParent"])
	frame:SetScript("OnMouseUp", function() ClosetGnome:KeybindingClicked(comboCopy, setCopy) end)
	self.db.char.keybindings[setCopy] = comboCopy
	SetBindingClick(comboCopy, frameName)

	self:UpdateDisplay()
end

function ClosetGnome:UnsetBinding(set)
	if not set or not self.db.char.keybindings[set] then return end
	local combo = self.db.char.keybindings[set]

	local frame = _G[keybindingFrame..combo]
	if frame then
		frame:SetScript("OnMouseUp", nil)
		frame = nil
	end

	SetBinding(combo) -- Unset the binding.
end

function ClosetGnome:ClearBinding(set)
	if not set or not self.db.char.keybindings[set] then return end

	self:UnsetBinding(set)

	local combo = self.db.char.keybindings[set]
	self:Print(string.format(L["Removing keybinding %s from set %s."], "|cffd9d919"..combo.."|r", "|cffd9d919"..set.."|r"))
	self.db.char.keybindings[set] = nil

	self:UpdateDisplay()
end

function ClosetGnome:KeybindingClicked(combo, set)
	if not set or not self:ValidateBinding(combo) then return end
	self:WearSet(set)
end

-------------------------------------------------------------------------------
-- Menu                                                                      --
-------------------------------------------------------------------------------

function ClosetGnome:UpdateSetMenu()
	self.OnMenuRequest.args.wear.args = {}
	self.OnMenuRequest.args.delete.args = {}
	self.OnMenuRequest.args.key.args = {}

	for k, v in pairs(self.db.char.set) do
		if not self:HideSetFromUI(k) then
			self.OnMenuRequest.args.wear.args[k:gsub(" ", "")] = {
				["type"] = "execute",
				["name"] = k,
				["desc"] = string.format(L["Equip %s."], k),
				["func"] = function() ClosetGnome:WearSet(k) end,
				["icon"] = self.db.char.icons[k],
			}
		end
		self.OnMenuRequest.args.delete.args[k:gsub(" ", "")] = {
			["type"] = "execute",
			["name"] = k,
			["desc"] = string.format(L["Deletes the equipment set %s."], k),
			["func"] = function() ClosetGnome:DeleteSet(k, false) end,
			["icon"] = self.db.char.icons[k],
		}
		self.OnMenuRequest.args.key.args[k:gsub(" ", "")] = {
			["type"] = "text",
			["name"] = k,
			["desc"] = string.format(L["Assign a keybinding to %s (or empty to clear)."], k),
			["validate"] = "keybinding",
			["get"] = function() return ClosetGnome.db.char.keybindings[k] end,
			["set"] = function(v)
				if v == nil then
					ClosetGnome:ClearBinding(k)
				else
					ClosetGnome:RegisterBinding(v, k)
				end
			end,
			["icon"] = self.db.char.icons[k],
		}
	end
end

-------------------------------------------------------------------------------
-- Addon methods                                                             --
-------------------------------------------------------------------------------

local function OnEnter()
	if type(PaperDollItemSlotButton_OnEnter) == "function" then
		this = _G[this:GetName():gsub("ClosetGnome", "")]
		PaperDollItemSlotButton_OnEnter()
	end
end

local function OnLeave()
	_G[this:GetName():gsub("ClosetGnome", "")].updateTooltip = nil
	GameTooltip:Hide()
	ResetCursor()
end

local function OnClick()
	local slot = this:GetName():gsub("ClosetGnome", "")
	if not IsModifierKeyDown() and CursorHasItem() and type(PaperDollItemSlotButton_OnClick) == "function" then
		PaperDollItemSlotButton_OnClick(arg1)
	elseif IsControlKeyDown() then
		-- Don't want an empty slot as icon.
		local id = GetInventorySlotInfo(slot:sub(10))
		local currentItem = ClosetGnome:ItemNameFromSlot(id)
		if currentItem == false then return end
		if activeSlots["icon"] then
			local oldSlot = activeSlots["icon"]
			activeSlots[oldSlot] = true
			_G[oldSlot.."Texture"]:SetVertexColor(0, 255, 0)
		end
		activeSlots["icon"] = slot
		activeSlots[slot] = true
		_G[slot.."Texture"]:SetVertexColor(0, 0, 255)
	else
		activeSlots[slot] = not activeSlots[slot]
		if activeSlots[slot] then
			if activeSlots["icon"] == slot then
				_G[slot.."Texture"]:SetVertexColor(0, 0, 255)
			else
				_G[slot.."Texture"]:SetVertexColor(0, 255, 0)
			end
		else
			_G[slot.."Texture"]:SetVertexColor(255, 0, 0)
		end
	end
end

-- Opens the character frame and lets the user uncheck the items he doesn't
-- want.
function ClosetGnome:CreateSet()
	activeSlots = {}
	for _, item in ipairs(slots) do
		local gslot = _G["Character" .. item.slot .. "Slot"]
		if not item.frame and not item.texture and gslot then
			-- We used to just :GetScript("OnClick") on the existing slot,
			-- and save that so that we could override it and put it back
			-- after we close the character frame, but this is not possible
			-- in TBC with the protected code stuff. WoW will complain that
			-- the script is tainted, and the normal slots won't work again
			-- after adding a set, so we create our own frame that we set on
			-- top of the existing one now.
			local button = CreateFrame("Button", gslot:GetName() .. "ClosetGnome", gslot)
			button:SetAlpha(0)
			button:SetHeight(gslot:GetHeight())
			button:SetWidth(gslot:GetWidth())
			button:SetPoint("CENTER", gslot, "CENTER", 0, 1)
			button:SetScript("OnEnter", OnEnter)
			button:SetScript("OnLeave", OnLeave)
			button:SetScript("OnClick", OnClick)
			button:Hide()

			local texture = gslot:CreateTexture(gslot:GetName() .. "Texture", "OVERLAY")
			texture:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
			texture:SetVertexColor(0, 255, 0)
			texture:SetBlendMode("ADD")
			texture:SetAlpha(0.75)
			texture:SetHeight(68)
			texture:SetWidth(68)
			texture:SetPoint("CENTER", gslot, "CENTER", 0, 1)
			texture:Hide()

			item.frame = button
			item.texture = texture
		end
		if item.frame and item.texture and gslot then
			if item.inactive then
				activeSlots[gslot:GetName()] = false
				item.texture:SetVertexColor(255, 0, 0)
			else
				activeSlots[gslot:GetName()] = true
				item.texture:SetVertexColor(0, 255, 0)
			end
			item.frame:EnableMouse(true)
			item.frame:Show()
			item.texture:Show()
		end
	end

	-- Hook Fizzle to prevent it from styling the slots while we add a set.
	if IsAddOnLoaded("Fizzle") and type(Fizzle) == "table" and type(Fizzle.UpdateItems) == "function" then
		self:Hook(Fizzle, "UpdateItems", "Noop")
	end

	ShowUIPanel(CharacterFrame)
	CharacterFrame_ShowSubFrame("PaperDollFrame")
	OpenAllBags(1)
	self:Hook("CharacterFrame_OnHide", true)
	self:Hook("CharacterFrameTab_OnClick", "Noop", true)
	self:Hook("PaperDollItemSlotButton_OnModifiedClick", "Noop", true)

	dewdrop:Close()
	StaticPopup_Show("ClosetGnomeAdd")
end

function ClosetGnome:CancelCreateSet()
	CloseAllBags()
	HideUIPanel(CharacterFrame)
	if self:IsHooked("CharacterFrame_OnHide") then
		self:Unhook("CharacterFrame_OnHide")
	end
	if Fizzle and self:IsHooked(Fizzle, "UpdateItems") then
		self:Unhook(Fizzle, "UpdateItems")
	end
	if self:IsHooked("PaperDollItemSlotButton_OnModifiedClick") then
		self:Unhook("PaperDollItemSlotButton_OnModifiedClick")
	end
end

function ClosetGnome:AddSetFromDoll(name)
	local slots = {}
	local icon = nil
	for slot, active in pairs(activeSlots) do
		if slot == "icon" then
			icon = active:sub(10)
		elseif active then
			table.insert(slots, slot:sub(10))
		end
	end
	self:AddSet(name, slots, icon)
	activeSlots = nil
	slots = nil
	icon = nil

	HideUIPanel(CharacterFrame)
	self:CharacterFrame_OnHide()
end

function ClosetGnome:AddSet(name, slots, iconSlot)
	local concatedName = name:gsub(" ", "")
	if not name or name == "" or concatedName == "" then
		self:Print(L["Please use a proper name for your set."])
		return
	end
	if self:HasSet(concatedName) then name = concatedName end

	local text = string.format(L["Added set: %s."], name)
	local event = "ClosetGnome_AddSet"
	if self:HasSet(name) then
		text = string.format(L["Updating set: %s."], name)
		event = "ClosetGnome_UpdateSet"
	end

	for i, slot in pairs(slots) do
		local id = GetInventorySlotInfo(slot)
		if not id then error(string.format("The given slot %q does not exist.", slot)) end
		local currentItem = self:ItemNameFromSlot(id)
		if not self.db.char.set[name] then self.db.char.set[name] = {} end
		self.db.char.set[name][id] = currentItem
	end
	if not self.db.char.set[name] then return end

	if iconSlot then
		local slotId = GetInventorySlotInfo(iconSlot)
		if not slotId then error(string.format("The given icon slot %q does not exist.", iconSlot)) end
		self.db.char.icons[name] = GetInventoryItemTexture("player", slotId)
	end

	lastEquippedSet = name
	self:TriggerEvent(event, name)
	self:Print(text)

	self:UpdateSetMenu()
	self:UpdateDisplay()
end

function ClosetGnome:DeleteSet(name, verified)
	if not verified and not IsShiftKeyDown() then
		queuedForDelete = name
		StaticPopup_Show("ClosetGnomeDelete", name)
		return
	end

	queuedForDelete = nil
	if self.db.char.set[name] then
		for i in pairs(self.db.char.set[name]) do
			self.db.char.set[name][i] = nil
		end
		self.db.char.set[name] = nil
		self.db.char.icons[name] = nil
		self:ClearBinding(name)

		if lastEquippedSet == name then
			lastEquippedSet = nil
		end

		self:TriggerEvent("ClosetGnome_DeleteSet", name)
		self:Print(string.format(L["Deleted set: %s."], name))
		self:UpdateSetMenu()
		self:UpdateDisplay()
	end
end

function ClosetGnome:WearSet(name)
	ClearCursor()
	if CursorHasItem() or CursorHasSpell() or UnitIsDeadOrGhost("player") then return false end

	local set = self.db.char.set[name]
	if not set then return false end

	dewdrop:Close()

	if self:IsSetFullyEquipped(name) then
		lastEquippedSet = name
		return true
	end

	if UnitAffectingCombat("player") then
		if self.db.char.alwaysWeapons then
			deequipQueue = nil
			for slot, item in pairs(set) do
				if slot == 16 or slot == 17 or slot == 18 then
					local currentItem = self:ItemNameFromSlot(slot)
					if item ~= currentItem then
						self:EquipItem(slot, item)
					end
				end
			end
			self:ProcessDeequipQueue()
			lastEquippedSet = name
			self:UpdateDisplay()
		end

		if self:IsSetFullyEquipped(name) then
			self:TriggerEvent("ClosetGnome_WearSet", name)
			return true
		end

		self:TriggerEvent("ClosetGnome_PartlyWearSet", name)
		if not self:IsEventRegistered("PLAYER_REGEN_ENABLED") then
			self:RegisterEvent("PLAYER_REGEN_ENABLED")
		end
		self:Print(string.format(L["In combat: %s queued for swap."], name))
		queuedSet = name
		return true
	end

	if self:IsEventRegistered("PLAYER_REGEN_ENABLED") then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end

	queuedSet = nil

	if self.db.profile.quips then
		-- I don't want the addon name in front for this one.
		-- Thanks to MrPlow for the color :)
		DEFAULT_CHAT_FRAME:AddMessage(self:GetQuip(name))
	end

	deequipQueue = nil
	for slot, item in pairs(set) do
		local currentItem = self:ItemNameFromSlot(slot)
		if item ~= currentItem then
			self:EquipItem(slot, item)
		end
	end
	self:ProcessDeequipQueue()
	lastEquippedSet = name
	self:TriggerEvent("ClosetGnome_WearSet", name)

	self:UpdateDisplay()

	return true
end

-------------------------------------------------------------------------------
-- Utility                                                                   --
-------------------------------------------------------------------------------

function ClosetGnome:GetQuip(input)
	local rand = 1
	repeat rand = math.random(1, NUM_QUIPS) until L[rand] ~= nil
	return "|cffeda55f"..string.format(L[rand], "|cffd9d919"..input.."|cffeda55f").."|r"
end

function ClosetGnome:ItemNameFromSlot(slot)
	assert(slot, "Slot must be non-nil.")
	local slotId = tonumber(slot)
	if not slotId then slotId = GetInventorySlotInfo(slot) end
	if not slotId then error(string.format("Unknown inventory slot %q.", slot)) end
	local itemLink = GetInventoryItemLink("player", slotId)
	if not itemLink then return false end
	return itemLink
end

function ClosetGnome:IsSetFullyEquipped(name)
	local set = self.db.char.set[name]
	if not set or not name then return false end
	for slot, item in pairs(set) do
		local currentItem = self:ItemNameFromSlot(slot)
		if item ~= currentItem then return false end
	end
	return true
end

function ClosetGnome:GetItemSubtype(link)
	local _,_,_,_,_,_,subtype = GetItemInfo(link)
	return subtype
end

function ClosetGnome:IsNormalBag(bagId)
	if bagId == 0 or bagId == -1 then return true end
	local link = GetInventoryItemLink("player", ContainerIDToInventoryID(bagId))
	if not link then return false end
	local _, _, linkId = link:find("item:(%d+)")
	if not linkId then return false end
	local bagType = self:GetItemSubtype(linkId)
	if bagType and bagType == L["Bag"] then return true end
	return false
end

function ClosetGnome:LocateFreeSlot()
	for theBag = NUM_BAG_FRAMES, 0, -1 do
		if self:IsNormalBag(theBag) then
			if not slotLocks then slotLocks = {} end
			if not slotLocks[theBag] then slotLocks[theBag] = {} end
			local numSlot = GetContainerNumSlots(theBag)
			for theSlot = 1, numSlot, 1 do
				if not slotLocks[theBag][theSlot] then
					local texture = GetContainerItemInfo(theBag, theSlot)
					if not texture then
						return theBag, theSlot
					end
				end
			end
		end
	end
	return nil
end

function ClosetGnome:ProcessDeequipQueue()
	if not deequipQueue then return end
	if not slotLocks then slotLocks = {} end
	for i, slot in pairs(deequipQueue) do
		PickupInventoryItem(slot)
		local toBag, toSlot = self:LocateFreeSlot()
		if toBag ~= nil then
			slotLocks[toBag][toSlot] = true
			PickupContainerItem(toBag, toSlot)
		else
			AutoEquipCursorItem()
			break
		end
	end
	deequipQueue = nil
	slotLocks = nil
end

function ClosetGnome:EquipItem(slot, item, secondTry)
	if item == false then
		-- We need to find a free slot in the inventory for this item.
		-- If there's anything in the slot.
		local hasItem = self:ItemNameFromSlot(slot)
		if hasItem ~= false then
			if not deequipQueue then deequipQueue = {} end
			table.insert(deequipQueue, slot)
		end
	else
		local bagNum, slotNum = self:FindItem(item)
		if bagNum > -1 then
			PickupContainerItem(bagNum, slotNum)
			if slot < 11 or slot == 15 or slot == 19 then
				AutoEquipCursorItem()
			else
				EquipCursorItem(slot)
			end
		else
			if not secondTry and item then
				self:ScheduleEvent("cg"..slot..item, self.EquipItem, 0.1, self, slot, item, true)
			else
				self:Print(string.format(L["Couldn't find %s in your inventory."], tostring(item)))
			end
		end
	end
end

function ClosetGnome:HasSet(name)
	return self.db.char.set[name] ~= nil
end

function ClosetGnome:FindItem(item)
	for i = NUM_BAG_FRAMES, 0, -1 do
		for j = GetContainerNumSlots(i), 1, -1 do
			local link = GetContainerItemLink(i, j)
			if link and item == link then return i, j end
		end
	end
	return -1
end

-------------------------------------------------------------------------------
-- Hooks                                                                     --
-------------------------------------------------------------------------------

-- Only hooked if we are showing the paper doll frame with set adding
function ClosetGnome:CharacterFrame_OnHide()
	for _, item in ipairs(slots) do
		if item.frame then
			item.frame:EnableMouse(false)
			item.frame:Hide()
			item.texture:Hide()
		end
	end

	StaticPopup_Hide("ClosetGnomeAdd")
	if self:IsHooked("CharacterFrame_OnHide") then
		self:Unhook("CharacterFrame_OnHide")
	end
	if self:IsHooked("CharacterFrameTab_OnClick") then
		self:Unhook("CharacterFrameTab_OnClick")
	end
	if Fizzle and self:IsHooked(Fizzle, "UpdateItems") then
		self:Unhook(Fizzle, "UpdateItems")
	end
	if self:IsHooked("PaperDollItemSlotButton_OnModifiedClick") then
		self:Unhook("PaperDollItemSlotButton_OnModifiedClick")
	end
end

function ClosetGnome:Noop() --[[ noop ]] end

-------------------------------------------------------------------------------
-- Events                                                                    --
-------------------------------------------------------------------------------

function ClosetGnome:AceEvent_FullyInitialized()
	-- Loop through the current sets and check if we have any of them equipped.
	for k,v in pairs(self.db.char.set) do
		local ret = self:IsSetFullyEquipped(k)
		if ret then
			lastEquippedSet = k
			self:TriggerEvent("ClosetGnome_WearSet", k)
			break
		end
	end
	self:UpdateDisplay()

	self:Print("|cff00ff00ClosetGnome still has 1 or 2 issues that has to be resolved before release, but I consider it usable. Please direct any comments to me on IRC or to the forum post.|r")
end

function ClosetGnome:PLAYER_REGEN_ENABLED()
	if not queuedSet then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	self:WearSet(queuedSet)
end

-------------------------------------------------------------------------------
-- FuBarPlugin                                                               --
-------------------------------------------------------------------------------

function ClosetGnome:OnTextUpdate()
	if lastEquippedSet then
		self:SetText(self:GetSetDisplayName(lastEquippedSet))
	else
		self:SetText(L["ClosetGnome"])
	end
	if self.db.char.icons[lastEquippedSet] then
		self:SetIcon(self.db.char.icons[lastEquippedSet])
	else
		self:SetIcon(defaultIcon)
	end
end

function ClosetGnome:OnDoubleClick()
	self:CreateSet()
end

-- Hook this method and return true for any sets you do not want listed in the
-- tablet.
function ClosetGnome:HideSetFromUI(set)
	return false
end

-- Hook to provide for example coloring of different sets in the tablet.
function ClosetGnome:GetSetDisplayName(set)
	return set
end

function ClosetGnome:OnTooltipUpdate()
	local cat = nil
	local function clickfunc(name)
		if IsControlKeyDown() then
			local set = self.db.char.set[name]
			DressUpModel:Undress()
			for i = 1, 19 do
				local item = set[i]
				if i ~= 18 and item ~= false then
					DressUpItemLink(item)
				end
			end
		else
			self:WearSet(name)
		end
	end

	local sets = {}
	for k in pairs(self.db.char.set) do table.insert(sets, k) end
	table.sort(sets)

	for i, set in ipairs(sets) do
		if not self:HideSetFromUI(set) then
			if not cat then
				cat = tablet:AddCategory(
					"columns", 2,
					"text", L["Available sets"],
					"showWithoutChildren", false,
					"child_hasCheck", true,
					"child_checked", true,
					"child_text2R", 0.5,
					"child_text2G", 0.5,
					"child_text2B", 0.5
				)
			end
			local displayName = self:GetSetDisplayName(set)
			if lastEquippedSet == set then
				displayName = string.format(L["%s |cffd9d919*|r"], self:GetSetDisplayName(set))
			end
			cat:AddLine(
				"text", displayName,
				"func", clickfunc,
				"arg1", set,
				"checkIcon", self.db.char.icons[set] or "",
				"text2", self.db.char.keybindings[set] or ""
			)
		end
	end
	sets = nil

	if cat ~= nil then
		tablet:SetHint(L["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. The last equipped set is indicated by a star."])
	else
		tablet:SetHint(L["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."])
	end
end

